"""
This module is a collection of tools that are useful
to manage modules and odoo environment.

The submodule modules can be used to manipulate and
search existing module in a given environment.

The module ``odoo_tools.odoo`` gives a simple api
to work with an odoo environment. It's an api
that consume the collection of function in other
modules of this package.
"""
